# -*- coding: utf-8 -*-
# @Author: wangkun
# @Time: 2022/5/16
import datetime
import os
import sys
import time
import requests
import urllib3
sys.path.append(os.getcwd())
from main.common import Common
from main.feishu_lib import Feishu
from main.publish import Publish

proxies = {"http": None, "https": None}


class HourList:
    # 配置微信
    wechat_sheet = Feishu.get_values_batch("hour", "xiaoniangao", "dzcWHw")
    hour_x_b3_traceid = wechat_sheet[2][1]
    hour_x_token_id = wechat_sheet[3][1]
    hour_referer = wechat_sheet[4][1]
    hour_uid = wechat_sheet[5][1]
    hour_token = wechat_sheet[6][1]

    # 小时级数据表
    hour_sheet = Feishu.get_values_batch("hour", "xiaoniangao", "ba0da4")

    # 过滤敏感词
    @classmethod
    def sensitive_words(cls, log_type):
        # 敏感词库列表
        word_list = []
        # 从云文档读取所有敏感词，添加到词库列表
        time.sleep(1)
        lists = Feishu.get_values_batch(log_type, "xiaoniangao", "DRAnZh")
        for i in lists:
            for j in i:
                # 过滤空的单元格内容
                if j is None:
                    pass
                else:
                    word_list.append(j)
        return word_list

    # 视频ID过滤字母
    @classmethod
    def sensitive_videoid_words(cls):
        # 字母列表
        words_list = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s",
                      "t", "u", "v", "w", "x", "y", "z",
                      "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S",
                      "T", "U", "V", "W", "X", "Y", "Z"]
        return words_list

    # 基础门槛规则
    @staticmethod
    def download_rule(d_duration, d_width, d_height, d_play_cnt, d_like_cnt, d_share_cnt, d_send_time):
        """
        下载视频的基本规则
        :param d_duration: 时长
        :param d_width: 宽
        :param d_height: 高
        :param d_play_cnt: 播放量
        :param d_like_cnt: 点赞量
        :param d_share_cnt: 分享量
        :param d_send_time: 发布时间
        :return: 满足规则，返回 True；反之，返回 False
        """
        # 视频时长
        if int(float(d_duration)) >= 40:
            # 宽或高
            if int(d_width) >= 0 or int(d_height) >= 0:
                # 播放量
                if int(d_play_cnt) >= 4000:
                    # 点赞量
                    if int(d_like_cnt) >= 0:
                        # 分享量
                        if int(d_share_cnt) >= 0:
                            # 发布时间 <= 10 天
                            if int(time.time()) - int(d_send_time)/1000 <= 864000:
                                return True
                            else:
                                return False
                        else:
                            return False
                    else:
                        return False
                else:
                    return False
            return False
        return False

    # 检查是否有今日的上升榜日期
    @classmethod
    def check_hour_list_data(cls, log_type, date):
        # 判断J1单元格的日期是否为今天
        time.sleep(1)
        if cls.hour_sheet[0][11] != date:
            # 插入3列 L1:N1，并写入日期和时间数据
            values = [[date], ["10:00", "15:00", "20:00"]]
            time.sleep(1)
            Feishu.insert_columns(log_type, "xiaoniangao", "ba0da4", "COLUMNS", 11, 14)
            time.sleep(1)
            Feishu.update_values(log_type, "xiaoniangao", "ba0da4",  "L1:N2", values)
            time.sleep(1)
            Feishu.merge_cells(log_type, "xiaoniangao", "ba0da4", "L1:N1")
            Common.logger(log_type).info("插入今天日期成功")
        else:
            Common.logger(log_type).info("今日上升榜日期已存在")

    # 获取列表
    @classmethod
    def get_hour_list_feeds(cls, log_type):
        """
        1.从列表获取视频，7 天内，播放量>=5000
        2.时长 1-10min
        3.每天10:00、15:00、20:00 把符合规则的视频，写入云文档
        https://w42nne6hzg.feishu.cn/sheets/shtcnYxiyQ1wLklo1W5Kdqc9cGh?sheet=ba0da4
        """
        url = "https://kapi.xiaoniangao.cn/trends/get_recommend_trends"
        headers = {
            "x-b3-traceid": cls.hour_x_b3_traceid,
            "X-Token-Id": cls.hour_x_token_id,
            "uid": cls.hour_uid,
            "content-type": "application/json",
            "Accept-Encoding": "gzip,compress,br,deflate",
            "User-Agent": 'Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X)'
                          ' AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 '
                          'MicroMessenger/8.0.20(0x18001432) NetType/WIFI Language/zh_CN',
            "Referer": cls.hour_referer
        }
        data = {
            "log_params": {
                "page": "discover_rec",
                "common": {
                    "brand": "iPhone",
                    "device": "iPhone 11",
                    "os": "iOS 14.7.1",
                    "weixinver": "8.0.20",
                    "srcver": "2.24.2",
                    "net": "wifi",
                    "scene": 1089
                }
            },
            "qs": "imageMogr2/gravity/center/rotate/$/thumbnail/!750x500r/crop/750x500/interlace/1/format/jpg",
            "h_qs": "imageMogr2/gravity/center/rotate/$/thumbnail/!80x80r/crop/80x80/interlace/1/format/jpg",
            "share_width": 625,
            "share_height": 500,
            "ext": {
                "fmid": 0,
                "items": {}
            },
            "app": "xng",
            "rec_scene": "discover_rec",
            "log_common_params": {
                "e": [{
                    "data": {
                        "page": "discoverIndexPage",
                        "topic": "recommend"
                    },
                    "ab": {}
                }],
                "ext": {
                    "brand": "iPhone",
                    "device": "iPhone 11",
                    "os": "iOS 14.7.1",
                    "weixinver": "8.0.20",
                    "srcver": "2.24.3",
                    "net": "wifi",
                    "scene": "1089"
                },
                "pj": "1",
                "pf": "2",
                "session_id": "7bcce313-b57d-4305-8d14-6ebd9a1bad29"
            },
            "refresh": False,
            "token": cls.hour_token,
            "uid": cls.hour_uid,
            "proj": "ma",
            "wx_ver": "8.0.20",
            "code_ver": "3.62.0"
        }
        try:
            urllib3.disable_warnings()
            r = requests.post(url=url, headers=headers, json=data, proxies=proxies, verify=False)
            if "data" not in r.json():
                Common.logger(log_type).warning("获取视频feeds错误:{}", r.text)
            elif "list" not in r.json()["data"]:
                Common.logger(log_type).warning("获取视频feeds无数据，休眠10s:{}", r.json()["data"])
            else:
                # 视频列表数据
                feeds = r.json()["data"]["list"]
                for i in range(len(feeds)):
                    # 标题
                    if "title" in feeds[i]:
                        video_title = feeds[i]["title"].strip().replace("\n", "") \
                            .replace("/", "").replace("\r", "").replace("#", "") \
                            .replace(".", "。").replace("\\", "").replace("&NBSP", "") \
                            .replace(":", "").replace("*", "").replace("？", "") \
                            .replace("?", "").replace('"', "").replace("<", "") \
                            .replace(">", "").replace("|", "").replace(" ", "")
                    else:
                        video_title = 0

                    # 视频 ID
                    if "vid" in feeds[i]:
                        video_id = feeds[i]["vid"]
                    else:
                        video_id = 0

                    # 播放量
                    if "play_pv" in feeds[i]:
                        video_play_cnt = feeds[i]["play_pv"]
                    else:
                        video_play_cnt = 0

                    # 点赞量
                    if "favor" in feeds[i]:
                        video_like_cnt = feeds[i]["favor"]["total"]
                    else:
                        video_like_cnt = 0

                    # 分享量
                    if "share" in feeds[i]:
                        video_share_cnt = feeds[i]["share"]
                    else:
                        video_share_cnt = 0

                    # # 评论量
                    # if "comment_count" in feeds[i]:
                    #     video_comment_cnt = feeds[i]["comment_count"]
                    # else:
                    #     video_comment_cnt = 0

                    # 时长
                    if "du" in feeds[i]:
                        video_duration = int(feeds[i]["du"] / 1000)
                    else:
                        video_duration = 0

                    # 宽和高
                    if "w" or "h" in feeds[i]:
                        video_width = feeds[i]["w"]
                        video_height = feeds[i]["h"]
                    else:
                        video_width = 0
                        video_height = 0

                    # 发布时间
                    if "t" in feeds[i]:
                        video_send_time = feeds[i]["t"]
                    else:
                        video_send_time = 0

                    # 用户名 / 头像
                    if "user" in feeds[i]:
                        user_name = feeds[i]["user"]["nick"].strip().replace("\n", "") \
                            .replace("/", "").replace("快手", "").replace(" ", "") \
                            .replace(" ", "").replace("&NBSP", "").replace("\r", "")
                        head_url = feeds[i]["user"]["hurl"]
                    else:
                        user_name = 0
                        head_url = 0

                    # 用户 ID
                    profile_id = feeds[i]["id"]

                    # 用户 mid
                    profile_mid = feeds[i]["user"]["mid"]

                    # 视频封面
                    if "url" in feeds[i]:
                        cover_url = feeds[i]["url"]
                    else:
                        cover_url = 0

                    # 视频播放地址
                    if "v_url" in feeds[i]:
                        video_url = feeds[i]["v_url"]
                    else:
                        video_url = 0

                    Common.logger(log_type).info("标题:{}", video_title)
                    Common.logger(log_type).info("视频ID:{}", video_id)
                    Common.logger(log_type).info("播放量:{}", video_play_cnt)
                    # Common.logger(log_type).info("点赞量:{}", video_like_cnt)
                    # Common.logger(log_type).info("分享量:{}", video_share_cnt)
                    # Common.logger(log_type).info("评论数:{}", video_comment_cnt)
                    Common.logger(log_type).info("时长:{}秒", video_duration)
                    # Common.logger(log_type).info("宽高:{}*{}", video_width, video_height)
                    Common.logger(log_type).info(
                        "视频发布时间:{}", time.strftime(
                            "%Y-%m-%d %H:%M:%S", time.localtime(int(video_send_time) / 1000)))
                    Common.logger(log_type).info("用户名:{}", user_name)
                    # Common.logger(log_type).info("用户头像:{}", head_url)
                    # Common.logger(log_type).info("封面:{}", cover_url)
                    Common.logger(log_type).info("播放地址:{}", video_url)

                    # 过滤无效视频
                    if video_title == 0 or video_id == 0 or video_duration == 0 \
                            or video_send_time == 0 or user_name == 0 or head_url == 0 \
                            or cover_url == 0 or video_url == 0:
                        Common.logger(log_type).warning("无效视频")

                    elif cls.download_rule(video_duration, video_width, video_height, video_play_cnt,
                                           video_like_cnt, video_share_cnt, video_send_time) is False:
                        Common.logger(log_type).info("不满足基础门槛规则")

                    # 过滤敏感词
                    elif any(word if word in video_title else False for word in cls.sensitive_words(log_type)) is True:
                        Common.logger(log_type).info("视频已中敏感词:{}".format(video_title))
                        time.sleep(1)

                    # 从云文档中去重:https://w42nne6hzg.feishu.cn/sheets/shtcnYxiyQ1wLklo1W5Kdqc9cGh?sheet=yatRv2
                    elif video_id in [j for i in Feishu.get_values_batch(log_type, "xiaoniangao", "yatRv2") for j in i]:
                        Common.logger(log_type).info("该视频已下载:{}", video_title)
                        time.sleep(1)

                    # 从云文档去重:https://w42nne6hzg.feishu.cn/sheets/shtcnYxiyQ1wLklo1W5Kdqc9cGh?sheet=ba0da4
                    elif video_id in [j for i in Feishu.get_values_batch("hour", "xiaoniangao", "ba0da4") for j in i]:
                        Common.logger(log_type).info("该视频已保存过:{}", video_title)
                        time.sleep(1)
                    else:
                        Common.logger(log_type).info("该视频未下载，添加至feeds中:{}".format(video_title))
                        # feeds工作表，插入空行
                        time.sleep(1)
                        Feishu.insert_columns(log_type, "xiaoniangao", "ba0da4", "ROWS", 2, 3)

                        # 获取当前时间
                        get_feeds_time = int(time.time())
                        # 看一看云文档，工作表中写入数据
                        values = [[profile_id,
                                   profile_mid,
                                   video_id,
                                   video_title,
                                   user_name,
                                   video_duration,
                                   cover_url,
                                   video_url,
                                   time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(int(video_send_time) / 1000)),
                                   str(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(get_feeds_time))),
                                   video_play_cnt]]
                        # 等待 1s，防止操作云文档太频繁，导致报错
                        time.sleep(1)
                        Feishu.update_values(log_type, "xiaoniangao", "ba0da4", "A3:K3", values)

        except Exception as e:
            Common.logger(log_type).error("获取小时榜视频列表异常:{}", e)

    # 更新小时榜数据
    @classmethod
    def update_hour_list_data(cls, log_type, today, yesterday, before_yesterday):
        """
        更新小时榜数据
        """
        try:
            update_hour_sheet = Feishu.get_values_batch("hour", "xiaoniangao", "ba0da4")
            if len(update_hour_sheet) == 2:
                Common.logger(log_type).info("当前工作表无数据")
            else:
                for i in range(2, len(update_hour_sheet)+1):
                    Common.logger(log_type).info("更新第:{}行视频信息", i+1)

                    # 略过空行
                    if update_hour_sheet[i][0] is None \
                            or update_hour_sheet[i][1] is None or update_hour_sheet[i][2] is None:
                        Common.logger(log_type).info("空行，略过")
                    else:
                        # 视频标题
                        v_title = update_hour_sheet[i][3]
                        Common.logger(log_type).info("video_title:{}", v_title)

                        # 视频 ID
                        v_id = update_hour_sheet[i][2]
                        Common.logger(log_type).info("video_id:{}", v_id)

                        # profile_id，用户 ID
                        p_id = update_hour_sheet[i][0]
                        Common.logger(log_type).info("profile_id:{}", p_id)

                        # profile_mid
                        p_mid = update_hour_sheet[i][1]
                        Common.logger(log_type).info("profile_mid:{}", p_mid)

                        # 抓取时的播放量
                        v_play_cnt = update_hour_sheet[i][10]
                        Common.logger(log_type).info("video_play_cnt:{}", v_play_cnt)

                        # 抓取时间
                        v_upload_time = update_hour_sheet[i][9]
                        Common.logger(log_type).info("video_send_time:{}", v_upload_time)
                        # 抓取时间的时间戳格式（秒为单位）
                        v_time = int(time.mktime(time.strptime(v_upload_time, "%Y-%m-%d %H:%M:%S")))

                        # 抓取时间:日期
                        upload_data = v_upload_time.split(" ")[0]
                        # 抓取时间:小时
                        upload_hour = v_upload_time.split(" ")[-1].split(":")[0]

                        url = "https://kapi.xiaoniangao.cn/profile/get_profile_by_id"
                        headers = {
                            "x-b3-traceid": cls.hour_x_b3_traceid,
                            "X-Token-Id": cls.hour_x_token_id,
                            "uid": cls.hour_uid,
                            "content-type": "application/json",
                            "Accept-Encoding": "gzip,compress,br,deflate",
                            "User-Agent": 'Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X)'
                                          ' AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 '
                                          'MicroMessenger/8.0.20(0x18001432) NetType/WIFI Language/zh_CN',
                            "Referer": cls.hour_referer
                        }
                        data = {
                            "play_src": "1",
                            "profile_id": int(p_id),
                            "profile_mid": int(p_mid),
                            "qs": "imageMogr2/gravity/center/rotate/$/thumbnail/"
                                  "!400x400r/crop/400x400/interlace/1/format/jpg",
                            "h_qs": "imageMogr2/gravity/center/rotate/$/thumbnail"
                                    "/!80x80r/crop/80x80/interlace/1/format/jpg",
                            "share_width": 625,
                            "share_height": 500,
                            "no_comments": True,
                            "no_follow": True,
                            "vid": v_id,
                            "hot_l1_comment": True,
                            "token": cls.hour_token,
                            "uid": cls.hour_uid,
                            "proj": "ma",
                            "wx_ver": "8.0.20",
                            "code_ver": "3.62.0",
                            "log_common_params": {
                                "e": [{
                                    "data": {
                                        "page": "dynamicSharePage"
                                    }
                                }],
                                "ext": {
                                    "brand": "iPhone",
                                    "device": "iPhone 11",
                                    "os": "iOS 14.7.1",
                                    "weixinver": "8.0.20",
                                    "srcver": "2.24.3",
                                    "net": "wifi",
                                    "scene": "1089"
                                },
                                "pj": "1",
                                "pf": "2",
                                "session_id": "7bcce313-b57d-4305-8d14-6ebd9a1bad29"
                            }
                        }
                        try:
                            urllib3.disable_warnings()
                            r = requests.post(headers=headers, url=url, json=data, proxies=proxies, verify=False)
                            hour_play_cnt = r.json()["data"]["play_pv"]
                            Common.logger(log_type).info("视频详情，当前播放量:{}", hour_play_cnt)
                            # 固定时间获取符合规则的视频，写入云文档:https://w42nne6hzg.feishu.cn/sheets/shtcngRPoDYAi24x52j2nDuHMih?sheet=ba0da4
                            update_hour = datetime.datetime.now()
                            if int(time.time()) - v_time >= 172800:
                                Common.logger(log_type).info("抓取时间超过 2 天")
                                return
                            elif upload_data == today and update_hour.hour == 10 and int(upload_hour) <= 10:
                                Common.logger(log_type).info("满足条件: 抓取日期为今天 and 当前时间:10点 and 抓取时间<=10点")

                                # 当天 10:00 视频播放量
                                ten_hour_play_cnt = hour_play_cnt
                                Common.logger(log_type).info("当天 10:00 视频播放量:{}", ten_hour_play_cnt)

                                # 10:00 的上升榜写入数据
                                values = int(ten_hour_play_cnt) - int(v_play_cnt)
                                time.sleep(1)
                                Feishu.update_values(
                                    log_type, "xiaoniangao", "ba0da4",
                                    "L" + str(i+1) + ":" + "L" + str(i+1), [[values]])
                                Common.logger(log_type).info("10:00数据更新成功:{}", values)

                            elif upload_data == today and update_hour.hour == 15 and int(upload_hour) <= 10:
                                Common.logger(log_type).info("满足条件: 抓取日期为今天 and 当前时间:15点 and 抓取时间<=10点")

                                # 当天 15:00 视频播放量
                                fifteen_hour_play_cnt = hour_play_cnt
                                Common.logger(log_type).info("当天 15:00 视频播放量:{}", fifteen_hour_play_cnt)

                                # 当天 10:00 上升的数据
                                if update_hour_sheet[i][11] is None:
                                    ten_up_cnt = 0
                                else:
                                    ten_up_cnt = update_hour_sheet[i][11]

                                # 15:00 的上升榜写入数据
                                values = int(fifteen_hour_play_cnt) - (int(v_play_cnt) + int(ten_up_cnt))
                                time.sleep(1)
                                Feishu.update_values(
                                    log_type, "xiaoniangao", "ba0da4",
                                    "M" + str(i+1) + ":" + "M" + str(i+1), [[values]])
                                Common.logger(log_type).info("15:00数据更新成功:{}", values)

                            elif upload_data == today and update_hour.hour == 15 and 10 < int(upload_hour) <= 15:
                                Common.logger(log_type).info("满足条件: 抓取日期为今天 and 当前时间:15点 and 10<抓取时间<=15点")

                                # 当天 15:00 视频播放量
                                fifteen_hour_play_cnt = hour_play_cnt
                                Common.logger(log_type).info("当天 15:00 视频播放量:{}", fifteen_hour_play_cnt)

                                # 15:00 的上升榜写入数据
                                values = int(fifteen_hour_play_cnt) - int(v_play_cnt)
                                time.sleep(1)
                                Feishu.update_values(
                                    log_type, "xiaoniangao", "ba0da4",
                                    "M" + str(i+1) + ":" + "M" + str(i+1), [[values]])
                                Common.logger(log_type).info("15:00数据更新成功:{}", values)

                            elif upload_data == today and update_hour.hour == 20 and int(upload_hour) <= 10:
                                Common.logger(log_type).info("满足条件: 抓取日期为今天 and 当前时间:20点 and 抓取时间<=10点")

                                # 当天 20:00 视频播放量
                                twenty_hour_play_cnt = hour_play_cnt
                                Common.logger(log_type).info("当天 20:00 视频播放量:{}", twenty_hour_play_cnt)

                                # 当天 10:00 上升的数据
                                if update_hour_sheet[i][11] is None:
                                    ten_up_cnt = 0
                                else:
                                    ten_up_cnt = update_hour_sheet[i][11]

                                # 当天 15:00 上升的数据
                                if update_hour_sheet[i][12] is None:
                                    fifteen_up_cnt = 0
                                else:
                                    fifteen_up_cnt = update_hour_sheet[i][12]

                                # 20:00 的上升榜写入数据
                                values = int(twenty_hour_play_cnt) - (
                                        int(v_play_cnt) + int(ten_up_cnt) + int(fifteen_up_cnt))
                                time.sleep(1)
                                Feishu.update_values(
                                    log_type, "xiaoniangao", "ba0da4",
                                    "N" + str(i+1) + ":" + "N" + str(i+1), [[values]])
                                Common.logger(log_type).info("20:00数据更新成功:{}", values)

                            elif upload_data == today and update_hour.hour == 20 and 10 < int(upload_hour) <= 15:
                                Common.logger(log_type).info("满足条件: 抓取日期为今天 and 当前时间:20点 and 10<抓取时间<=15点")

                                # 当天 20:00 视频播放量
                                twenty_hour_play_cnt = hour_play_cnt
                                Common.logger(log_type).info("当天 20:00 视频播放量:{}", twenty_hour_play_cnt)

                                # 当天 15:00 上升的数据
                                if update_hour_sheet[i][12] is None:
                                    fifteen_up_cnt = 0
                                else:
                                    fifteen_up_cnt = update_hour_sheet[i][12]

                                # 20:00 的上升榜写入数据
                                values = int(twenty_hour_play_cnt) - (int(v_play_cnt) + int(fifteen_up_cnt))
                                time.sleep(1)
                                Feishu.update_values(
                                    log_type, "xiaoniangao", "ba0da4",
                                    "N" + str(i+1) + ":" + "N" + str(i+1), [[values]])
                                Common.logger(log_type).info("20:00数据更新成功:{}", values)

                            elif upload_data == today and update_hour.hour == 20 and 15 < int(upload_hour) <= 20:
                                Common.logger(log_type).info("满足条件: 抓取日期为今天 and 当前时间:20点 and 15<抓取时间<=20点")

                                # 当天 20:00 视频播放量
                                twenty_hour_play_cnt = hour_play_cnt
                                Common.logger(log_type).info("当天 20:00 视频播放量:{}", twenty_hour_play_cnt)

                                # 20:00 的上升榜写入数据
                                values = int(twenty_hour_play_cnt) - int(v_play_cnt)
                                time.sleep(1)
                                Feishu.update_values(
                                    log_type, "xiaoniangao", "ba0da4",
                                    "N" + str(i+1) + ":" + "N" + str(i+1), [[values]])
                                Common.logger(log_type).info("20:00数据更新成功:{}", values)

                            elif (upload_data == yesterday or upload_data == before_yesterday)\
                                    and update_hour.hour == 10:
                                Common.logger(log_type).info("满足条件: 抓取时间小于今天 and 当前时间:10点")

                                # 当天 10:00 视频播放量
                                ten_hour_play_cnt = hour_play_cnt
                                Common.logger(log_type).info("当天 10:00 视频播放量:{}", ten_hour_play_cnt)

                                # 10:00 的上升榜写入数据
                                values = int(ten_hour_play_cnt) - int(v_play_cnt)
                                time.sleep(1)
                                Feishu.update_values(
                                    log_type, "xiaoniangao", "ba0da4",
                                    "L" + str(i+1) + ":" + "L" + str(i+1), [[values]])
                                Common.logger(log_type).info("10:00数据更新成功:{}", values)

                            elif (upload_data == yesterday or upload_data == before_yesterday)\
                                    and update_hour.hour == 15:
                                Common.logger(log_type).info("满足条件: 抓取时间小于今天 and 当前时间:15点")

                                # 当天 15:00 视频播放量
                                fifteen_hour_play_cnt = hour_play_cnt
                                Common.logger(log_type).info("当天 15:00 视频播放量:{}", fifteen_hour_play_cnt)

                                # 当天 10:00 上升的数据
                                if update_hour_sheet[i][11] is None:
                                    ten_up_cnt = 0
                                else:
                                    ten_up_cnt = update_hour_sheet[i][11]

                                # 15:00 的上升榜写入数据
                                values = int(fifteen_hour_play_cnt) - (int(v_play_cnt) + int(ten_up_cnt))
                                time.sleep(1)
                                Feishu.update_values(
                                    log_type, "xiaoniangao", "ba0da4",
                                    "M" + str(i+1) + ":" + "M" + str(i+1), [[values]])
                                Common.logger(log_type).info("15:00数据更新成功:{}", values)

                            elif (upload_data == yesterday or upload_data == before_yesterday)\
                                    and update_hour.hour == 20:
                                Common.logger(log_type).info("满足条件: 抓取时间小于今天 and 当前时间:20点")

                                # 当天 20:00 视频播放量
                                twenty_hour_play_cnt = hour_play_cnt
                                Common.logger(log_type).info("当天 20:00 视频播放量:{}", twenty_hour_play_cnt)

                                # 当天 10:00 上升的数据
                                if update_hour_sheet[i][11] is None:
                                    ten_up_cnt = 0
                                else:
                                    ten_up_cnt = update_hour_sheet[i][11]

                                # 当天 15:00 上升的数据
                                if update_hour_sheet[i][12] is None:
                                    fifteen_up_cnt = 0
                                else:
                                    fifteen_up_cnt = update_hour_sheet[i][12]

                                # 20:00 的上升榜写入数据
                                values = int(twenty_hour_play_cnt) - (
                                        int(v_play_cnt) + int(ten_up_cnt) + int(fifteen_up_cnt))
                                time.sleep(1)
                                Feishu.update_values(
                                    log_type, "xiaoniangao", "ba0da4",
                                    "N" + str(i+1) + ":" + "N" + str(i+1), [[values]])
                                Common.logger(log_type).info("20:00数据更新成功:{}", values)

                        except Exception as e:
                            Common.logger(log_type).error("视频详情:{}，异常:{}", v_title, e)
        except Exception as e:
            Common.logger(log_type).error("获取小时榜数据异常:{}", e)

    # 下载/上传
    @classmethod
    def download_and_publish(cls, log_type):
        """
        1.从云文档中去重: https://w42nne6hzg.feishu.cn/sheets/shtcnYxiyQ1wLklo1W5Kdqc9cGh?sheet=yatRv2
        2.从云文档中下载符合规则的视频:https://w42nne6hzg.feishu.cn/sheets/shtcnYxiyQ1wLklo1W5Kdqc9cGh?sheet=ba0da4
            2.1 当日 10:00 or 15:00 or 20:00 视频播放量上升 > 5000
            2.2 当日 10:00 and 15:00 视频播放量上升 > 2000
            2.3 当日 15:00 and 20:00 视频播放量上升 > 2000
            2.4 昨日 20:00 and 今日 10:00 视频播放量上升 > 2000
        3.上传
        """
        try:
            download_hour_sheet = Feishu.get_values_batch("hour", "xiaoniangao", "ba0da4")
            if len(download_hour_sheet) == 2:
                Common.logger(log_type).info("当前工作表无数据")
            else:
                for i in range(2, len(download_hour_sheet)+1):
                    Common.logger(log_type).info("分析第:{}行视频信息是否符合下载规则", i+1)

                    # 略过空行
                    if download_hour_sheet[i][0] is None \
                            or download_hour_sheet[i][1] is None or download_hour_sheet[i][2] is None:
                        Common.logger(log_type).info("空行，略过")
                    else:
                        # 今日 10:00 数据上升量
                        if download_hour_sheet[i][11] is None:
                            ten_cnt = 0
                        else:
                            ten_cnt = download_hour_sheet[i][11]
                        # 今日 15:00 数据上升量
                        if download_hour_sheet[i][12] is None:
                            fifteen_cnt = 0
                        else:
                            fifteen_cnt = download_hour_sheet[i][12]
                        # 今日 20:00 数据上升量
                        if download_hour_sheet[i][13] is None:
                            twenty_cnt = 0
                        else:
                            twenty_cnt = download_hour_sheet[i][13]
                        # 昨日 20:00 数据上升量
                        if download_hour_sheet[i][16] is None:
                            yesterday_twenty_cnt = 0
                        else:
                            yesterday_twenty_cnt = download_hour_sheet[i][16]

                        # 视频标题
                        v_title = download_hour_sheet[i][3]
                        Common.logger(log_type).info("video_title:{}", v_title)

                        # 视频 ID
                        v_id = download_hour_sheet[i][2]
                        Common.logger(log_type).info("video_id:{}", v_id)

                        # profile_id，用户 ID
                        p_id = download_hour_sheet[i][0]

                        # 视频时长
                        v_duration = download_hour_sheet[i][5]

                        # profile_mid
                        p_mid = download_hour_sheet[i][1]

                        Common.logger(log_type).info("10:00 / 15:00 / 20:00 上升量: {} / {} / {}",
                                                     ten_cnt, fifteen_cnt, twenty_cnt)

                        # 抓取时间
                        v_upload_time = download_hour_sheet[i][9]
                        v_send_time = int(time.mktime(time.strptime(v_upload_time, "%Y-%m-%d %H:%M:%S")))

                        # 播放量
                        v_play_cnt = download_hour_sheet[i][10]

                        # # 判断视频 ID 长度，不大于 13 位
                        # if len(str(v_id)) > 13:
                        #     Common.logger(log_type).info("视频ID长度大于13位:{}", v_id)
                        #
                        # el
                        if int(time.time()) - int(v_send_time) >= 259200:
                            Common.logger(log_type).info("抓取时间超过 3 天")
                            return

                        # 判断视频时长：1-10min
                        elif int(v_duration) < 40:
                            Common.logger(log_type).info("视频时长小于 40s")

                        # # 过滤带字母的视频ID
                        # elif any(word if word in v_id else False for word in cls.sensitive_videoid_words()) is True:
                        #     Common.logger(log_type).info("视频ID带字母:{}".format(v_id))

                        # 从云文档中去重:https://w42nne6hzg.feishu.cn/sheets/shtcnYxiyQ1wLklo1W5Kdqc9cGh?sheet=yatRv2
                        elif v_id in [j for i in Feishu.get_values_batch(log_type, "xiaoniangao", "yatRv2") for j in i]:
                            Common.logger(log_type).info("该视频已下载:{}", v_title)

                        # 播放量大于 100000，直接下载
                        elif int(v_play_cnt) >= 100000:
                            Common.logger(log_type).info("播放量:{} >= 100000，满足下载规则，开始下载视频", v_play_cnt)

                            try:
                                url = "https://kapi.xiaoniangao.cn/profile/get_profile_by_id"
                                headers = {
                                    "x-b3-traceid": cls.hour_x_b3_traceid,
                                    "X-Token-Id": cls.hour_x_token_id,
                                    "uid": cls.hour_uid,
                                    "content-type": "application/json",
                                    "Accept-Encoding": "gzip,compress,br,deflate",
                                    "User-Agent": 'Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X)'
                                                  ' AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 '
                                                  'MicroMessenger/8.0.20(0x18001432) NetType/WIFI Language/zh_CN',
                                    "Referer": cls.hour_referer
                                }
                                data = {
                                    "play_src": "1",
                                    "profile_id": int(p_id),
                                    "profile_mid": int(p_mid),
                                    "qs": "imageMogr2/gravity/center/rotate/$/thumbnail/"
                                          "!400x400r/crop/400x400/interlace/1/format/jpg",
                                    "h_qs": "imageMogr2/gravity/center/rotate/$/thumbnail"
                                            "/!80x80r/crop/80x80/interlace/1/format/jpg",
                                    "share_width": 625,
                                    "share_height": 500,
                                    "no_comments": True,
                                    "no_follow": True,
                                    "vid": v_id,
                                    "hot_l1_comment": True,
                                    "token": cls.hour_token,
                                    "uid": cls.hour_uid,
                                    "proj": "ma",
                                    "wx_ver": "8.0.20",
                                    "code_ver": "3.62.0",
                                    "log_common_params": {
                                        "e": [{
                                            "data": {
                                                "page": "dynamicSharePage"
                                            }
                                        }],
                                        "ext": {
                                            "brand": "iPhone",
                                            "device": "iPhone 11",
                                            "os": "iOS 14.7.1",
                                            "weixinver": "8.0.20",
                                            "srcver": "2.24.3",
                                            "net": "wifi",
                                            "scene": "1089"
                                        },
                                        "pj": "1",
                                        "pf": "2",
                                        "session_id": "7bcce313-b57d-4305-8d14-6ebd9a1bad29"
                                    }
                                }
                                urllib3.disable_warnings()
                                r = requests.post(headers=headers, url=url, json=data, proxies=proxies, verify=False)
                                hour_play_cnt = r.json()["data"]["play_pv"]
                                hour_cover_url = r.json()["data"]["url"]
                                hour_video_url = r.json()["data"]["v_url"]
                                hour_video_duration = r.json()["data"]["du"]
                                hour_video_comment_cnt = r.json()["data"]["comment_count"]
                                hour_video_like_cnt = r.json()["data"]["favor"]["total"]
                                hour_video_share_cnt = r.json()["data"]["share"]
                                hour_video_width = r.json()["data"]["w"]
                                hour_video_height = r.json()["data"]["h"]
                                hour_video_resolution = str(hour_video_width) + "*" + str(hour_video_height)
                                hour_video_send_time = r.json()["data"]["t"]
                                hour_user_name = r.json()["data"]["user"]["nick"]
                                hour_head_url = r.json()["data"]["user"]["hurl"]
                                Common.logger(log_type).info("视频详情，当前播放量:{}", hour_play_cnt)

                                # 下载封面
                                Common.download_method(log_type, "cover", v_title, hour_cover_url)
                                # 下载视频
                                Common.download_method(log_type, "video", v_title, hour_video_url)
                                # 保存视频信息至 "./videos/{download_video_title}/info.txt"
                                with open("./videos/" + v_title + "/" + "info.txt", "a", encoding="UTF-8") as f_a:
                                    f_a.write(str(v_id) + "\n" +
                                              str(v_title) + "\n" +
                                              str(int(int(hour_video_duration) / 1000)) + "\n" +
                                              str(hour_play_cnt) + "\n" +
                                              str(hour_video_comment_cnt) + "\n" +
                                              str(hour_video_like_cnt) + "\n" +
                                              str(hour_video_share_cnt) + "\n" +
                                              str(hour_video_resolution) + "\n" +
                                              str(hour_video_send_time) + "\n" +
                                              str(hour_user_name) + "\n" +
                                              str(hour_head_url) + "\n" +
                                              str(hour_video_url) + "\n" +
                                              str(hour_cover_url) + "\n" +
                                              str("90747742180aeb22c0fe3a3c6a38f3d9"))
                                Common.logger(log_type).info("==========视频信息已保存至info.txt==========")

                                # 上传视频
                                Common.logger(log_type).info("开始上传视频")
                                our_video_id = Publish.upload_and_publish(log_type, "prod", "up")
                                our_video_link = "https://admin.piaoquantv.com/cms/post-detail/" + str(
                                    our_video_id) + "/info"
                                Common.logger(log_type).info("视频上传完成:{}", v_title)
                                # 上传完成时间
                                upload_time = int(time.time())

                                # 保存视频信息到云文档
                                Common.logger(log_type).info("添加视频到云文档:{}", v_title)
                                # 插入空行
                                time.sleep(1)
                                Feishu.insert_columns(log_type, "xiaoniangao", "yatRv2", "ROWS", 1, 2)
                                # 视频信息写入云文档
                                values = [[time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(int(upload_time))),
                                           "小时级上升榜",
                                           v_id,
                                           v_title,
                                           our_video_link,
                                           hour_play_cnt,
                                           hour_video_comment_cnt,
                                           hour_video_like_cnt,
                                           hour_video_share_cnt,
                                           int(hour_video_duration) / 1000,
                                           hour_video_resolution,
                                           time.strftime("%Y-%m-%d %H:%M:%S",
                                                         time.localtime(int(hour_video_send_time) / 1000)),
                                           hour_user_name,
                                           p_id,
                                           p_mid,
                                           hour_head_url,
                                           hour_cover_url,
                                           hour_video_url]]
                                time.sleep(1)
                                Feishu.update_values(log_type, "xiaoniangao", "yatRv2", "F2:W2", values)

                                # 保存视频信息到监控表
                                Common.logger(log_type).info("添加视频到监控表:{}", v_title)
                                # 插入空行
                                time.sleep(1)
                                Feishu.insert_columns(log_type, "monitor", "N7e2yI", "ROWS", 1, 2)
                                # 视频信息写入监控表
                                values = [[time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(int(upload_time))),
                                           v_id,
                                           v_title,
                                           our_video_link,
                                           p_id,
                                           p_mid,
                                           hour_user_name,
                                           int(hour_video_duration) / 1000,
                                           time.strftime("%Y/%m/%d %H:%M:%S",
                                                         time.localtime(int(hour_video_send_time) / 1000)),
                                           hour_play_cnt]]
                                time.sleep(1)
                                Feishu.update_values(log_type, "monitor", "N7e2yI", "F2:O2", values)
                            except Exception as e:
                                Common.logger(log_type).error("下载视频异常:{}", e)

                        # 上升榜判断逻辑，任意时间段上升量>=5000，连续两个时间段上升量>=2000
                        elif int(ten_cnt) >= 5000 or int(fifteen_cnt) >= 5000 or int(twenty_cnt) >= 5000:
                            Common.logger(log_type).info("10:00 or 15:00 or 20:00 数据上升量:{} or {} or {} >= 5000",
                                                         ten_cnt, fifteen_cnt, twenty_cnt)
                            Common.logger(log_type).info("满足下载规则，开始下载视频")

                            try:
                                url = "https://kapi.xiaoniangao.cn/profile/get_profile_by_id"
                                headers = {
                                    "x-b3-traceid": cls.hour_x_b3_traceid,
                                    "X-Token-Id": cls.hour_x_token_id,
                                    "uid": cls.hour_uid,
                                    "content-type": "application/json",
                                    "Accept-Encoding": "gzip,compress,br,deflate",
                                    "User-Agent": 'Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X)'
                                                  ' AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 '
                                                  'MicroMessenger/8.0.20(0x18001432) NetType/WIFI Language/zh_CN',
                                    "Referer": cls.hour_referer
                                }
                                data = {
                                    "play_src": "1",
                                    "profile_id": int(p_id),
                                    "profile_mid": int(p_mid),
                                    "qs": "imageMogr2/gravity/center/rotate/$/thumbnail/"
                                          "!400x400r/crop/400x400/interlace/1/format/jpg",
                                    "h_qs": "imageMogr2/gravity/center/rotate/$/thumbnail"
                                            "/!80x80r/crop/80x80/interlace/1/format/jpg",
                                    "share_width": 625,
                                    "share_height": 500,
                                    "no_comments": True,
                                    "no_follow": True,
                                    "vid": v_id,
                                    "hot_l1_comment": True,
                                    "token": cls.hour_token,
                                    "uid": cls.hour_uid,
                                    "proj": "ma",
                                    "wx_ver": "8.0.20",
                                    "code_ver": "3.62.0",
                                    "log_common_params": {
                                        "e": [{
                                            "data": {
                                                "page": "dynamicSharePage"
                                            }
                                        }],
                                        "ext": {
                                            "brand": "iPhone",
                                            "device": "iPhone 11",
                                            "os": "iOS 14.7.1",
                                            "weixinver": "8.0.20",
                                            "srcver": "2.24.3",
                                            "net": "wifi",
                                            "scene": "1089"
                                        },
                                        "pj": "1",
                                        "pf": "2",
                                        "session_id": "7bcce313-b57d-4305-8d14-6ebd9a1bad29"
                                    }
                                }
                                urllib3.disable_warnings()
                                r = requests.post(headers=headers, url=url, json=data, proxies=proxies, verify=False)
                                hour_play_cnt = r.json()["data"]["play_pv"]
                                hour_cover_url = r.json()["data"]["url"]
                                hour_video_url = r.json()["data"]["v_url"]
                                hour_video_duration = r.json()["data"]["du"]
                                hour_video_comment_cnt = r.json()["data"]["comment_count"]
                                hour_video_like_cnt = r.json()["data"]["favor"]["total"]
                                hour_video_share_cnt = r.json()["data"]["share"]
                                hour_video_width = r.json()["data"]["w"]
                                hour_video_height = r.json()["data"]["h"]
                                hour_video_resolution = str(hour_video_width) + "*" + str(hour_video_height)
                                hour_video_send_time = r.json()["data"]["t"]
                                hour_user_name = r.json()["data"]["user"]["nick"]
                                hour_head_url = r.json()["data"]["user"]["hurl"]
                                Common.logger(log_type).info("视频详情，当前播放量:{}", hour_play_cnt)

                                # 下载封面
                                Common.download_method(log_type, "cover", v_title, hour_cover_url)
                                # 下载视频
                                Common.download_method(log_type, "video", v_title, hour_video_url)
                                # 保存视频信息至 "./videos/{download_video_title}/info.txt"
                                with open("./videos/" + v_title + "/" + "info.txt", "a", encoding="UTF-8") as f_a:
                                    f_a.write(str(v_id) + "\n" +
                                              str(v_title) + "\n" +
                                              str(int(int(hour_video_duration) / 1000)) + "\n" +
                                              str(hour_play_cnt) + "\n" +
                                              str(hour_video_comment_cnt) + "\n" +
                                              str(hour_video_like_cnt) + "\n" +
                                              str(hour_video_share_cnt) + "\n" +
                                              str(hour_video_resolution) + "\n" +
                                              str(hour_video_send_time) + "\n" +
                                              str(hour_user_name) + "\n" +
                                              str(hour_head_url) + "\n" +
                                              str(hour_video_url) + "\n" +
                                              str(hour_cover_url) + "\n" +
                                              str("90747742180aeb22c0fe3a3c6a38f3d9"))
                                Common.logger(log_type).info("==========视频信息已保存至info.txt==========")

                                # 上传视频
                                Common.logger(log_type).info("开始上传视频")
                                our_video_id = Publish.upload_and_publish(log_type, "prod", "up")
                                our_video_link = "https://admin.piaoquantv.com/cms/post-detail/" + str(
                                    our_video_id) + "/info"
                                Common.logger(log_type).info("视频上传完成:{}", v_title)
                                # 上传完成时间
                                upload_time = int(time.time())

                                # 保存视频信息到云文档
                                Common.logger(log_type).info("添加视频到云文档:{}", v_title)
                                # 插入空行
                                time.sleep(1)
                                Feishu.insert_columns(log_type, "xiaoniangao", "yatRv2", "ROWS", 1, 2)
                                # 视频信息写入云文档
                                values = [[time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(int(upload_time))),
                                           "小时级上升榜",
                                           v_id,
                                           v_title,
                                           our_video_link,
                                           hour_play_cnt,
                                           hour_video_comment_cnt,
                                           hour_video_like_cnt,
                                           hour_video_share_cnt,
                                           int(hour_video_duration)/1000,
                                           hour_video_resolution,
                                           time.strftime("%Y-%m-%d %H:%M:%S",
                                                         time.localtime(int(hour_video_send_time) / 1000)),
                                           hour_user_name,
                                           p_id,
                                           p_mid,
                                           hour_head_url,
                                           hour_cover_url,
                                           hour_video_url]]
                                time.sleep(1)
                                Feishu.update_values(log_type, "xiaoniangao", "yatRv2", "F2:W2", values)

                                # 保存视频信息到监控表
                                Common.logger(log_type).info("添加视频到监控表:{}", v_title)
                                # 插入空行
                                time.sleep(1)
                                Feishu.insert_columns(log_type, "monitor", "N7e2yI", "ROWS", 1, 2)
                                # 视频信息写入监控表
                                values = [[time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(int(upload_time))),
                                           v_id,
                                           v_title,
                                           our_video_link,
                                           p_id,
                                           p_mid,
                                           hour_user_name,
                                           int(hour_video_duration)/1000,
                                           time.strftime("%Y/%m/%d %H:%M:%S",
                                                         time.localtime(int(hour_video_send_time) / 1000)),
                                           hour_play_cnt]]
                                time.sleep(1)
                                Feishu.update_values(log_type, "monitor", "N7e2yI", "F2:O2", values)
                            except Exception as e:
                                Common.logger(log_type).error("下载视频异常:{}", e)
                        elif int(ten_cnt) >= 2000 and int(fifteen_cnt) >= 2000:
                            Common.logger(log_type).info(
                                "10:00 and 15:00 数据上升量:{} and {} >= 2000", ten_cnt, fifteen_cnt)
                            Common.logger(log_type).info("满足下载规则，开始下载视频")

                            try:
                                url = "https://kapi.xiaoniangao.cn/profile/get_profile_by_id"
                                headers = {
                                    "x-b3-traceid": cls.hour_x_b3_traceid,
                                    "X-Token-Id": cls.hour_x_token_id,
                                    "uid": cls.hour_uid,
                                    "content-type": "application/json",
                                    "Accept-Encoding": "gzip,compress,br,deflate",
                                    "User-Agent": 'Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X)'
                                                  ' AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 '
                                                  'MicroMessenger/8.0.20(0x18001432) NetType/WIFI Language/zh_CN',
                                    "Referer": cls.hour_referer
                                }
                                data = {
                                    "play_src": "1",
                                    "profile_id": int(p_id),
                                    "profile_mid": int(p_mid),
                                    "qs": "imageMogr2/gravity/center/rotate/$/thumbnail/"
                                          "!400x400r/crop/400x400/interlace/1/format/jpg",
                                    "h_qs": "imageMogr2/gravity/center/rotate/$/thumbnail"
                                            "/!80x80r/crop/80x80/interlace/1/format/jpg",
                                    "share_width": 625,
                                    "share_height": 500,
                                    "no_comments": True,
                                    "no_follow": True,
                                    "vid": v_id,
                                    "hot_l1_comment": True,
                                    "token": cls.hour_token,
                                    "uid": cls.hour_uid,
                                    "proj": "ma",
                                    "wx_ver": "8.0.20",
                                    "code_ver": "3.62.0",
                                    "log_common_params": {
                                        "e": [{
                                            "data": {
                                                "page": "dynamicSharePage"
                                            }
                                        }],
                                        "ext": {
                                            "brand": "iPhone",
                                            "device": "iPhone 11",
                                            "os": "iOS 14.7.1",
                                            "weixinver": "8.0.20",
                                            "srcver": "2.24.3",
                                            "net": "wifi",
                                            "scene": "1089"
                                        },
                                        "pj": "1",
                                        "pf": "2",
                                        "session_id": "7bcce313-b57d-4305-8d14-6ebd9a1bad29"
                                    }
                                }
                                urllib3.disable_warnings()
                                r = requests.post(headers=headers, url=url, json=data, proxies=proxies, verify=False)
                                hour_play_cnt = r.json()["data"]["play_pv"]
                                hour_cover_url = r.json()["data"]["url"]
                                hour_video_url = r.json()["data"]["v_url"]
                                hour_video_duration = r.json()["data"]["du"]
                                hour_video_comment_cnt = r.json()["data"]["comment_count"]
                                hour_video_like_cnt = r.json()["data"]["favor"]["total"]
                                hour_video_share_cnt = r.json()["data"]["share"]
                                hour_video_width = r.json()["data"]["w"]
                                hour_video_height = r.json()["data"]["h"]
                                hour_video_resolution = str(hour_video_width) + "*" + str(hour_video_height)
                                hour_video_send_time = r.json()["data"]["t"]
                                hour_user_name = r.json()["data"]["user"]["nick"]
                                hour_head_url = r.json()["data"]["user"]["hurl"]
                                Common.logger(log_type).info("视频详情，当前播放量:{}", hour_play_cnt)

                                # 下载封面
                                Common.download_method(log_type, "cover", v_title, hour_cover_url)
                                # 下载视频
                                Common.download_method(log_type, "video", v_title, hour_video_url)
                                # 保存视频信息至 "./videos/{download_video_title}/info.txt"
                                with open("./videos/" + v_title + "/" + "info.txt", "a", encoding="UTF-8") as f_a:
                                    f_a.write(str(v_id) + "\n" +
                                              str(v_title) + "\n" +
                                              str(int(int(hour_video_duration) / 1000)) + "\n" +
                                              str(hour_play_cnt) + "\n" +
                                              str(hour_video_comment_cnt) + "\n" +
                                              str(hour_video_like_cnt) + "\n" +
                                              str(hour_video_share_cnt) + "\n" +
                                              str(hour_video_resolution) + "\n" +
                                              str(hour_video_send_time) + "\n" +
                                              str(hour_user_name) + "\n" +
                                              str(hour_head_url) + "\n" +
                                              str(hour_video_url) + "\n" +
                                              str(hour_cover_url) + "\n" +
                                              str("90747742180aeb22c0fe3a3c6a38f3d9"))
                                Common.logger(log_type).info("==========视频信息已保存至info.txt==========")

                                # 上传
                                Common.logger(log_type).info("开始上传视频")
                                our_video_id = Publish.upload_and_publish(log_type, "prod", "up")
                                our_video_link = "https://admin.piaoquantv.com/cms/post-detail/" + str(
                                    our_video_id) + "/info"
                                Common.logger(log_type).info("视频上传完成:{}", v_title)
                                # 上传完成时间
                                upload_time = int(time.time())

                                # 保存视频信息到云文档
                                Common.logger(log_type).info("添加视频到云文档:{}", v_title)
                                # 插入空行
                                time.sleep(1)
                                Feishu.insert_columns(log_type, "xiaoniangao", "yatRv2", "ROWS", 1, 2)
                                # 视频信息写入云文档
                                values = [[time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(int(upload_time))),
                                           "小时级上升榜",
                                           v_id,
                                           v_title,
                                           our_video_link,
                                           hour_play_cnt,
                                           hour_video_comment_cnt,
                                           hour_video_like_cnt,
                                           hour_video_share_cnt,
                                           int(hour_video_duration) / 1000,
                                           hour_video_resolution,
                                           time.strftime("%Y-%m-%d %H:%M:%S",
                                                         time.localtime(int(hour_video_send_time) / 1000)),
                                           hour_user_name,
                                           p_id,
                                           p_mid,
                                           hour_head_url,
                                           hour_cover_url,
                                           hour_video_url]]
                                time.sleep(1)
                                Feishu.update_values(log_type, "xiaoniangao", "yatRv2", "F2:W2", values)

                                # 保存视频信息到监控表
                                Common.logger(log_type).info("添加视频到监控表:{}", v_title)
                                # 插入空行
                                time.sleep(1)
                                Feishu.insert_columns(log_type, "monitor", "N7e2yI", "ROWS", 1, 2)
                                # 视频信息写入监控表
                                values = [[time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(int(upload_time))),
                                           v_id,
                                           v_title,
                                           our_video_link,
                                           p_id,
                                           p_mid,
                                           hour_user_name,
                                           int(hour_video_duration) / 1000,
                                           time.strftime("%Y/%m/%d %H:%M:%S",
                                                         time.localtime(int(hour_video_send_time) / 1000)),
                                           hour_play_cnt]]
                                time.sleep(1)
                                Feishu.update_values(log_type, "monitor", "N7e2yI", "F2:O2", values)
                            except Exception as e:
                                Common.logger(log_type).error("下载视频异常:{}", e)
                        elif int(fifteen_cnt) >= 2000 and int(twenty_cnt) >= 2000:
                            Common.logger(log_type).info(
                                "15:00 and 20:00 数据上升量:{} and {} >= 2000", fifteen_cnt, twenty_cnt)
                            Common.logger(log_type).info("满足下载规则，开始下载视频")

                            try:
                                url = "https://kapi.xiaoniangao.cn/profile/get_profile_by_id"
                                headers = {
                                    "x-b3-traceid": cls.hour_x_b3_traceid,
                                    "X-Token-Id": cls.hour_x_token_id,
                                    "uid": cls.hour_uid,
                                    "content-type": "application/json",
                                    "Accept-Encoding": "gzip,compress,br,deflate",
                                    "User-Agent": 'Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X)'
                                                  ' AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 '
                                                  'MicroMessenger/8.0.20(0x18001432) NetType/WIFI Language/zh_CN',
                                    "Referer": cls.hour_referer
                                }
                                data = {
                                    "play_src": "1",
                                    "profile_id": int(p_id),
                                    "profile_mid": int(p_mid),
                                    "qs": "imageMogr2/gravity/center/rotate/$/thumbnail/"
                                          "!400x400r/crop/400x400/interlace/1/format/jpg",
                                    "h_qs": "imageMogr2/gravity/center/rotate/$/thumbnail"
                                            "/!80x80r/crop/80x80/interlace/1/format/jpg",
                                    "share_width": 625,
                                    "share_height": 500,
                                    "no_comments": True,
                                    "no_follow": True,
                                    "vid": v_id,
                                    "hot_l1_comment": True,
                                    "token": cls.hour_token,
                                    "uid": cls.hour_uid,
                                    "proj": "ma",
                                    "wx_ver": "8.0.20",
                                    "code_ver": "3.62.0",
                                    "log_common_params": {
                                        "e": [{
                                            "data": {
                                                "page": "dynamicSharePage"
                                            }
                                        }],
                                        "ext": {
                                            "brand": "iPhone",
                                            "device": "iPhone 11",
                                            "os": "iOS 14.7.1",
                                            "weixinver": "8.0.20",
                                            "srcver": "2.24.3",
                                            "net": "wifi",
                                            "scene": "1089"
                                        },
                                        "pj": "1",
                                        "pf": "2",
                                        "session_id": "7bcce313-b57d-4305-8d14-6ebd9a1bad29"
                                    }
                                }
                                urllib3.disable_warnings()
                                r = requests.post(headers=headers, url=url, json=data, proxies=proxies, verify=False)
                                hour_play_cnt = r.json()["data"]["play_pv"]
                                hour_cover_url = r.json()["data"]["url"]
                                hour_video_url = r.json()["data"]["v_url"]
                                hour_video_duration = r.json()["data"]["du"]
                                hour_video_comment_cnt = r.json()["data"]["comment_count"]
                                hour_video_like_cnt = r.json()["data"]["favor"]["total"]
                                hour_video_share_cnt = r.json()["data"]["share"]
                                hour_video_width = r.json()["data"]["w"]
                                hour_video_height = r.json()["data"]["h"]
                                hour_video_resolution = str(hour_video_width) + "*" + str(hour_video_height)
                                hour_video_send_time = r.json()["data"]["t"]
                                hour_user_name = r.json()["data"]["user"]["nick"]
                                hour_head_url = r.json()["data"]["user"]["hurl"]
                                Common.logger(log_type).info("视频详情，当前播放量:{}", hour_play_cnt)

                                # 下载封面
                                Common.download_method(log_type, "cover", v_title, hour_cover_url)
                                # 下载视频
                                Common.download_method(log_type, "video", v_title, hour_video_url)
                                # 保存视频信息至 "./videos/{download_video_title}/info.txt"
                                with open("./videos/" + v_title + "/" + "info.txt", "a", encoding="UTF-8") as f_a:
                                    f_a.write(str(v_id) + "\n" +
                                              str(v_title) + "\n" +
                                              str(int(int(hour_video_duration) / 1000)) + "\n" +
                                              str(hour_play_cnt) + "\n" +
                                              str(hour_video_comment_cnt) + "\n" +
                                              str(hour_video_like_cnt) + "\n" +
                                              str(hour_video_share_cnt) + "\n" +
                                              str(hour_video_resolution) + "\n" +
                                              str(hour_video_send_time) + "\n" +
                                              str(hour_user_name) + "\n" +
                                              str(hour_head_url) + "\n" +
                                              str(hour_video_url) + "\n" +
                                              str(hour_cover_url) + "\n" +
                                              str("90747742180aeb22c0fe3a3c6a38f3d9"))
                                Common.logger(log_type).info("==========视频信息已保存至info.txt==========")

                                # 上传
                                Common.logger(log_type).info("开始上传视频")
                                our_video_id = Publish.upload_and_publish(log_type, "prod", "up")
                                our_video_link = "https://admin.piaoquantv.com/cms/post-detail/" + str(
                                    our_video_id) + "/info"
                                Common.logger(log_type).info("视频上传完成:{}", v_title)
                                # 上传完成时间
                                upload_time = int(time.time())

                                # 保存视频信息到云文档
                                Common.logger(log_type).info("添加视频到云文档:{}", v_title)
                                # 插入空行
                                time.sleep(1)
                                Feishu.insert_columns(log_type, "xiaoniangao", "yatRv2", "ROWS", 1, 2)
                                # 视频信息写入云文档
                                values = [[time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(int(upload_time))),
                                           "小时级上升榜",
                                           v_id,
                                           v_title,
                                           our_video_link,
                                           hour_play_cnt,
                                           hour_video_comment_cnt,
                                           hour_video_like_cnt,
                                           hour_video_share_cnt,
                                           int(hour_video_duration) / 1000,
                                           hour_video_resolution,
                                           time.strftime("%Y-%m-%d %H:%M:%S",
                                                         time.localtime(int(hour_video_send_time) / 1000)),
                                           hour_user_name,
                                           p_id,
                                           p_mid,
                                           hour_head_url,
                                           hour_cover_url,
                                           hour_video_url]]
                                time.sleep(1)
                                Feishu.update_values(log_type, "xiaoniangao", "yatRv2", "F2:W2", values)

                                # 保存视频信息到监控表
                                Common.logger(log_type).info("添加视频到监控表:{}", v_title)
                                # 插入空行
                                time.sleep(1)
                                Feishu.insert_columns(log_type, "monitor", "N7e2yI", "ROWS", 1, 2)
                                # 视频信息写入监控表
                                values = [[time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(int(upload_time))),
                                           v_id,
                                           v_title,
                                           our_video_link,
                                           p_id,
                                           p_mid,
                                           hour_user_name,
                                           int(hour_video_duration) / 1000,
                                           time.strftime("%Y/%m/%d %H:%M:%S",
                                                         time.localtime(int(hour_video_send_time) / 1000)),
                                           hour_play_cnt]]
                                time.sleep(1)
                                Feishu.update_values(log_type, "monitor", "N7e2yI", "F2:O2", values)
                            except Exception as e:
                                Common.logger(log_type).error("下载视频异常:{}", e)
                        elif int(yesterday_twenty_cnt) >= 2000 and int(ten_cnt) >= 2000:
                            Common.logger(log_type).info("昨日20:00 and 今日10:00 数据上升量:{} and {} >= 2000",
                                                         yesterday_twenty_cnt, ten_cnt)
                            Common.logger(log_type).info("满足下载规则，开始下载视频")

                            try:
                                url = "https://kapi.xiaoniangao.cn/profile/get_profile_by_id"
                                headers = {
                                    "x-b3-traceid": cls.hour_x_b3_traceid,
                                    "X-Token-Id": cls.hour_x_token_id,
                                    "uid": cls.hour_uid,
                                    "content-type": "application/json",
                                    "Accept-Encoding": "gzip,compress,br,deflate",
                                    "User-Agent": 'Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X)'
                                                  ' AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 '
                                                  'MicroMessenger/8.0.20(0x18001432) NetType/WIFI Language/zh_CN',
                                    "Referer": cls.hour_referer
                                }
                                data = {
                                    "play_src": "1",
                                    "profile_id": int(p_id),
                                    "profile_mid": int(p_mid),
                                    "qs": "imageMogr2/gravity/center/rotate/$/thumbnail/"
                                          "!400x400r/crop/400x400/interlace/1/format/jpg",
                                    "h_qs": "imageMogr2/gravity/center/rotate/$/thumbnail"
                                            "/!80x80r/crop/80x80/interlace/1/format/jpg",
                                    "share_width": 625,
                                    "share_height": 500,
                                    "no_comments": True,
                                    "no_follow": True,
                                    "vid": v_id,
                                    "hot_l1_comment": True,
                                    "token": cls.hour_token,
                                    "uid": cls.hour_uid,
                                    "proj": "ma",
                                    "wx_ver": "8.0.20",
                                    "code_ver": "3.62.0",
                                    "log_common_params": {
                                        "e": [{
                                            "data": {
                                                "page": "dynamicSharePage"
                                            }
                                        }],
                                        "ext": {
                                            "brand": "iPhone",
                                            "device": "iPhone 11",
                                            "os": "iOS 14.7.1",
                                            "weixinver": "8.0.20",
                                            "srcver": "2.24.3",
                                            "net": "wifi",
                                            "scene": "1089"
                                        },
                                        "pj": "1",
                                        "pf": "2",
                                        "session_id": "7bcce313-b57d-4305-8d14-6ebd9a1bad29"
                                    }
                                }
                                urllib3.disable_warnings()
                                r = requests.post(headers=headers, url=url, json=data, proxies=proxies, verify=False)
                                hour_play_cnt = r.json()["data"]["play_pv"]
                                hour_cover_url = r.json()["data"]["url"]
                                hour_video_url = r.json()["data"]["v_url"]
                                hour_video_duration = r.json()["data"]["du"]
                                hour_video_comment_cnt = r.json()["data"]["comment_count"]
                                hour_video_like_cnt = r.json()["data"]["favor"]["total"]
                                hour_video_share_cnt = r.json()["data"]["share"]
                                hour_video_width = r.json()["data"]["w"]
                                hour_video_height = r.json()["data"]["h"]
                                hour_video_resolution = str(hour_video_width) + "*" + str(hour_video_height)
                                hour_video_send_time = r.json()["data"]["t"]
                                hour_user_name = r.json()["data"]["user"]["nick"]
                                hour_head_url = r.json()["data"]["user"]["hurl"]
                                Common.logger(log_type).info("视频详情，当前播放量:{}", hour_play_cnt)

                                # 下载封面
                                Common.download_method(log_type, "cover", v_title, hour_cover_url)
                                # 下载视频
                                Common.download_method(log_type, "video", v_title, hour_video_url)
                                # 保存视频信息至 "./videos/{download_video_title}/info.txt"
                                with open("./videos/" + v_title + "/" + "info.txt", "a", encoding="UTF-8") as f_a:
                                    f_a.write(str(v_id) + "\n" +
                                              str(v_title) + "\n" +
                                              str(int(int(hour_video_duration) / 1000)) + "\n" +
                                              str(hour_play_cnt) + "\n" +
                                              str(hour_video_comment_cnt) + "\n" +
                                              str(hour_video_like_cnt) + "\n" +
                                              str(hour_video_share_cnt) + "\n" +
                                              str(hour_video_resolution) + "\n" +
                                              str(hour_video_send_time) + "\n" +
                                              str(hour_user_name) + "\n" +
                                              str(hour_head_url) + "\n" +
                                              str(hour_video_url) + "\n" +
                                              str(hour_cover_url) + "\n" +
                                              str("90747742180aeb22c0fe3a3c6a38f3d9"))
                                Common.logger(log_type).info("==========视频信息已保存至info.txt==========")

                                # 上传
                                Common.logger(log_type).info("开始上传视频")
                                our_video_id = Publish.upload_and_publish(log_type, "prod", "up")
                                our_video_link = "https://admin.piaoquantv.com/cms/post-detail/" + str(
                                    our_video_id) + "/info"
                                Common.logger(log_type).info("视频上传完成:{}", v_title)
                                # 上传完成时间
                                upload_time = int(time.time())

                                # 保存视频信息到云文档
                                Common.logger(log_type).info("添加视频到云文档:{}", v_title)
                                # 插入空行
                                time.sleep(1)
                                Feishu.insert_columns(log_type, "xiaoniangao", "yatRv2", "ROWS", 1, 2)
                                # 视频信息写入云文档
                                values = [[time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(int(upload_time))),
                                           "小时级上升榜",
                                           v_id,
                                           v_title,
                                           our_video_link,
                                           hour_play_cnt,
                                           hour_video_comment_cnt,
                                           hour_video_like_cnt,
                                           hour_video_share_cnt,
                                           int(hour_video_duration) / 1000,
                                           hour_video_resolution,
                                           time.strftime("%Y-%m-%d %H:%M:%S",
                                                         time.localtime(int(hour_video_send_time) / 1000)),
                                           hour_user_name,
                                           p_id,
                                           p_mid,
                                           hour_head_url,
                                           hour_cover_url,
                                           hour_video_url]]
                                time.sleep(1)
                                Feishu.update_values(log_type, "xiaoniangao", "yatRv2", "F2:W2", values)

                                # 保存视频信息到监控表
                                Common.logger(log_type).info("添加视频到监控表:{}", v_title)
                                # 插入空行
                                time.sleep(1)
                                Feishu.insert_columns(log_type, "monitor", "N7e2yI", "ROWS", 1, 2)
                                # 视频信息写入监控表
                                values = [[time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(int(upload_time))),
                                           v_id,
                                           v_title,
                                           our_video_link,
                                           p_id,
                                           p_mid,
                                           hour_user_name,
                                           int(hour_video_duration) / 1000,
                                           time.strftime("%Y/%m/%d %H:%M:%S",
                                                         time.localtime(int(hour_video_send_time) / 1000)),
                                           hour_play_cnt]]
                                time.sleep(1)
                                Feishu.update_values(log_type, "monitor", "N7e2yI", "F2:O2", values)
                            except Exception as e:
                                Common.logger(log_type).error("下载视频异常:{}", e)
                        else:
                            Common.logger(log_type).info("上升量不满足下载规则")
        except Exception as e:
            Common.logger(log_type).error("下载视频时，获取飞书信息异常:{}", e)


if __name__ == "__main__":
    hour_list = HourList()
    # hour_list.get_hour_list_feeds()
    # hour_list.del_null_rows("xiaoniangao", "ba0da4", 3)
    # hour_list.update_hour_list_data(
    #     today=datetime.datetime.now().strftime("%Y-%m-%d"),
    #     yesterday=(datetime.date.today() + datetime.timedelta(days=-1)).strftime("%Y-%m-%d"),
    #     before_yesterday=(datetime.date.today() + datetime.timedelta(days=-2)).strftime("%Y-%m-%d"))
    # hour_list.download_and_publish("hour")
    hour_list.update_hour_list_data("hour", "2022-07-01", "2022-06-30", "2022-06-29")
